"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Apidoc = exports.KoaApidoc = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const apidoc_core_1 = tslib_1.__importDefault(require("apidoc-core"));
const markdown_it_1 = tslib_1.__importDefault(require("markdown-it"));
const koa_1 = require("@augejs/koa");
const core_1 = require("@augejs/core");
const koa_static_cache_1 = tslib_1.__importDefault(require("koa-static-cache"));
const ConfigName = 'apidoc';
// https://apidocjs.com
function KoaApidoc(opts) {
    return function (target) {
        core_1.Metadata.decorate([
            core_1.Config({
                [ConfigName]: {
                    url: '/apidoc'
                }
            }),
            core_1.LifecycleOnInitHook(async (scanNode, next) => {
                const config = Object.assign(Object.assign(Object.assign({}, scanNode.context.rootScanNode.getConfig(ConfigName)), scanNode.getConfig(ConfigName)), opts);
                const md = markdown_it_1.default();
                let header = null;
                if (config.header) {
                    header = {
                        content: md.render(config.header)
                    };
                }
                let footer = null;
                if (config.footer) {
                    footer = {
                        content: md.render(config.footer)
                    };
                }
                // eslint-disable-next-line @typescript-eslint/no-empty-function
                function mockLog() { }
                apidoc_core_1.default.setLogger({
                    debug: mockLog,
                    verbose: mockLog,
                    info: mockLog,
                    warn: mockLog,
                    error: mockLog,
                });
                const contents = core_1.Metadata.getMetadata(Apidoc, Apidoc);
                const apiResults = Array.isArray(contents) ?
                    contents.map(content => {
                        const parsedResults = apidoc_core_1.default.parseSource(Buffer.from(content));
                        if (Array.isArray(parsedResults)) {
                            const apiResult = parsedResults[0];
                            if (apiResult && apiResult.local) {
                                apiResults.push(apiResult);
                            }
                        }
                    }) : [];
                const koa = scanNode.context.container.get(koa_1.KOA_WEB_SERVER_IDENTIFIER);
                const apiDataPath = path_1.default.join('/', config.url + '', 'api_data.js');
                const apiProjectPath = path_1.default.join('/', config.url + '', 'api_project.js');
                koa.use(async (ctx, next) => {
                    var _a, _b, _c;
                    if (ctx.url.startsWith(apiDataPath)) {
                        ctx.type = 'application/javascript';
                        ctx.body = `define({"api": ${JSON.stringify(apiResults)}})`;
                    }
                    else if (ctx.url.startsWith(apiProjectPath)) {
                        ctx.type = 'application/javascript';
                        ctx.body = `define(${JSON.stringify({
                            name: (_a = config.title) !== null && _a !== void 0 ? _a : 'apidoc',
                            version: (_b = config.version) !== null && _b !== void 0 ? _b : '0.0.1',
                            description: (_c = config.description) !== null && _c !== void 0 ? _c : '',
                            template: config.template,
                            header: header || undefined,
                            footer: footer || undefined,
                            order: config.order,
                        })})`;
                    }
                    await next();
                });
                koa.use(koa_static_cache_1.default({
                    maxAge: 60 * 60 * 24 * 30,
                    prefix: config.url,
                    dir: path_1.default.join(__dirname, 'apidoc')
                }));
                await next();
            })
        ], target);
    };
}
exports.KoaApidoc = KoaApidoc;
function Apidoc(content) {
    return () => {
        core_1.Metadata.defineInsertEndArrayMetadata(Apidoc, [content], Apidoc);
    };
}
exports.Apidoc = Apidoc;
